import 'package:plugin_codelab_new/models/report_sub_type_model.dart';
import 'package:plugin_codelab_new/models/report_type_model.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:number_display/number_display.dart';
import 'dart:async';
import 'package:intl/intl.dart';
import 'package:datetime_picker_formfield/datetime_picker_formfield.dart';
import 'package:plugin_codelab_new/bloc/feed_cubit.dart';
import 'package:plugin_codelab_new/models/post_model.dart';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:plugin_codelab_new/pages/my_feed.dart';
import 'package:plugin_codelab_new/pages/share_post_page.dart';
import 'package:plugin_codelab_new/widgets/loadingButtonIndicator.dart';
import 'package:plugin_codelab_new/widgets/post.dart';
import 'package:pusher_websocket_flutter/pusher.dart';
import 'dart:convert';

import '../environment_config.dart';

final lastDate = new DateTime(
  DateTime.now().year - 10,
  DateTime.now().month,
);

String selectedValue = lastDate.toString();

class HomePage extends StatefulWidget {
  const HomePage({
    Key key,
    @required this.token,
    @required this.firstName,
    @required this.lastName,
    @required this.about,
    @required this.profileImg,
  }) : super(key: key);

  final String token;
  final String firstName;
  final String lastName;
  final String about;
  final String profileImg;

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  FeedCubit feedCubit;
  UserModel currentUser;
  List<PostModel> postsList = [];
  List<PostModel> postsListFilter = [];
  List<FeedPost> postsListWidgets = [];
  bool newPostFlag = false;
  var _scrollController = ScrollController();
  Channel _channel;

  @override
  void initState() {
    postsListFilter = [];
    postsListWidgets = [];
    feedCubit = BlocProvider.of<FeedCubit>(context);
    getCurrentUser();
    getPostData();
    _initPusher();
    _scrollController.addListener(_onScroll);
    super.initState();
  }

  dispose() {
    super.dispose();
  }

  void _onScroll() {
    if (_scrollController.position.pixels ==
        _scrollController.position.maxScrollExtent) {
      getPostData();
    }
  }

  getPostData() async {
    await getLikeSubId();
    await _loadData().then(
      (data) {
        setState(
          () {
            postsListFilter = data;
          },
        );
      },
    );
  }

  getCurrentUser() async {
    await feedCubit.feedRepository
        .authenticateUser(
            "Hadas",
            "Levi",
            "CTO",
            "https://res.cloudinary.com/phonedo/image/upload/v1614869100/shir_fdnec1.jpg",
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJpYXQiOjE2MTQxNjA0NDcsImV4cCI6MTYxNjY2MDQ0Nywicm9sZXMiOlsiUk9MRV9VU0VSIiwiUk9MRV9CVVNJTkVTU19BQ0NPVU5UX0FETUlOIl0sImlkIjoiN2EzM2FkYTMtMmFiMi00ZDRhLWIwOTgtNzkyZmRkMzM0NTE4IiwidHlwZSI6IkJ1c2luZXNzQWNjb3VudFVzZXIiLCJYQXBpVG9rZW4iOiJCQS01YzYyNmNlMWVmM2U1M2U4MjM0Y2YzOTcwOTk1MzA5MCJ9.TLjKkXatKB3BXp5M2Kghn6dZ6p2uoK5mgDyOMGPqBUDsyPQ6WIoU2baEQ5U2kClVU0zO1ouyskbRQcq7LHQE6oTupbGtlJ_HcGiq_7fh5WUfNFyKAOMRUG5cLtDOXCNF_mzRmDotLOpFBvBWLkUJmVrRD-bolZPH4t3EuWgasgbXgrcOEg3axdjgNqjoTo-fxvg1nrhi4MgSGvxyVKy92KTbSOk2xZ0zePMlAjpP5TMiNCQtlLWZeOBidl7g7BkcLYQ-NZ-fTCryQ2edZ_XscIDXFcPSwPh8KZchiNCaEVATRoXFtSQMGeixPV4gw76DGwpBkmCPbg4vZFt2RC_-292YIMxIZuxubYApTl-9Ul2tSOGQKzuQNy-93gZk4Ofkurv4SEcvqCm2sLHiVnhLjzVjmovtcUT9FlkJFoWJ5ahRsHY23VBZ2ibiwBhZljcql5bFCSp-TL62Nc2gIRo-MFDrOS_NOZgm7WvKIozLDWDc11EX0_iPGF3LccP7fKOP2PD60Nyi7WXCMY4JWLAg9edsjKQECjPoEMqW0rjzdNDTPwM-Tg51HoDTauieGCy91GnLP-q-BdSvxesbxkwSizQm_qONuTBmqufiHh7m5dQ7EGj1eazsLFx76S3_EH7eR0U9S1dUCmT8kg7DF6cRC39AJK-T7Rn3dPp9mlP96bo")
        .then(
      (data) {
        setState(
          () {
            currentUser = data;
          },
        );
      },
    );
  }

  Future<void> _initPusher() async {
    try {
      await Pusher.init(
          EnvironmentConfig.PUSHER_ENV, PusherOptions(cluster: 'ap2'));
    } catch (e) {
      print(e);
    }

    Pusher.connect(onConnectionStateChange: (val) {
      print(val.currentState);
    }, onError: (err) {
      print(err.message);
    });

    _channel = await Pusher.subscribe('post');

    _channel.bind('create_post', (onEvent) {
      List<dynamic> newPost = json.decode(onEvent.data);
      List<PostModel> postsList =
          newPost.map((i) => PostModel.fromJson(i)).toList();
      if (mounted) {
        this.setState(() {
          postsListFilter.insert(0, postsList[0]);
          newPostFlag = true;
        });
      }
    });

    _channel.bind('remove_post', (onEvent) {
      int postId = int.parse(onEvent.data);
      List<PostModel> posts = [];
      postsListFilter.forEach((post) {
        if (post.id != postId) {
          posts.add(post);
        }
      });
      if (mounted) {
        this.setState(() {
          postsListFilter = posts;
          print(postsListFilter[0].title);
        });
      }
    });
  }

  String filterDate = DateTime.now().toString();
  final display = createDisplay(
    length: 4,
    separator: ',',
    decimal: 2,
  );
  double defaultScreenWidth = 393.0;
  double defaultScreenHeight = 534.0;

  List<ReportTypeModel> reportsTypeList = [];
  List<ReportSubTypeModel> reportsSubTypeList = [];
  int likeId;
  int dislikeId;
  int saveId;

  void getPostsListFilter() {
    List<PostModel> postsListTemp = [];

    for (var i = 0; i < postsList.length; i++) {
      if (filter(
          createdAt: postsList[i].createdAt, selectedDate: selectedValue)) {
        postsListTemp.add(postsList[i]);
      }
    }
    setState(
      () {
        postsListFilter = [];
        for (var i = 0; i < postsListTemp.length; i++) {
          postsListFilter.add(postsListTemp[i]);
        }
      },
    );
  }

  Future getLikeSubId() async {
    reportsSubTypeList = await feedCubit.feedRepository.fetchAllSubType();
    for (var i = 0; i < reportsSubTypeList.length; i++) {
      if (likeId == null || dislikeId == null || saveId == null) {
        if (reportsSubTypeList[i].type == "Like") {
          likeId = reportsSubTypeList[i].id;
        } else if (reportsSubTypeList[i].type == "Dislike") {
          dislikeId = reportsSubTypeList[i].id;
        } else if (reportsSubTypeList[i].type == "Save") {
          saveId = reportsSubTypeList[i].id;
        }
      } else {
        break;
      }
    }
  }

  bool filter({@required DateTime createdAt, @required String selectedDate}) {
    DateTime fromDate = createdAt;
    DateTime toDate = new DateFormat("yyyy-MM-dd hh:mm:ss").parse(selectedDate);
    if (fromDate.isAfter(toDate) || toDate == fromDate) {
      return true;
    }
    return false;
  }

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(defaultScreenWidth, defaultScreenHeight),
      allowFontScaling: false,
      builder: () => LayoutBuilder(
        builder: (_, BoxConstraints constraints) {
          return Scaffold(
            appBar: PreferredSize(
                preferredSize: Size.fromHeight(70.0),
                child: AppBar(
                  backgroundColor: Color(0xFF083356),
                  leading: GestureDetector(
                      onTap: () {
                        Navigator.pop(context);
                      },
                      child: GestureDetector(
                          onTap: () {
                            Navigator.push(
                                context,
                                MaterialPageRoute(
                                    builder: (context) => MyFeedPage(
                                          currentUser: currentUser,
                                          saveId: saveId,
                                          likeId: likeId,
                                          dislikeId: dislikeId,
                                        )));
                          },
                          child: currentUser != null
                              ? Container(
                                  alignment: Alignment.bottomRight,
                                  child: ClipOval(
                                      child: Image.network(
                                    currentUser.profileImg,
                                    width: ScreenUtil().setWidth(40),
                                    height: ScreenUtil().setWidth(40),
                                    fit: BoxFit.cover,
                                  )))
                              : Icon(Icons.account_circle_outlined,
                                  size: 35, color: Colors.white))),
                  title: Padding(
                    padding: EdgeInsets.only(
                        left: ScreenUtil().setWidth(50),
                        top: ScreenUtil().setWidth(10)),
                    child: Text(
                      "Phone.do Feed",
                      style: TextStyle(color: Colors.white),
                    ),
                  ),
                )),
            body: postsListFilter.length < 1
                ? Center(child: LoadingIndicatorDark())
                : Container(
                    child: SafeArea(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.stretch,
                        children: [
                          _buildDateField(),
                          Expanded(
                            child: Stack(
                              children: [
                                RefreshIndicator(
                                  onRefresh: () {
                                    return _loadData();
                                  },
                                  child: ListView.separated(
                                    cacheExtent: 9999,
                                    controller: _scrollController,
                                    itemCount: postsListFilter.length,
                                    itemBuilder:
                                        (BuildContext context, int index) {
                                      PostModel post = postsListFilter[index];
                                      List<String> images =
                                          postsListFilter[index]
                                              .attachedUrl
                                              .split(",");
                                      return Container(
                                        decoration: BoxDecoration(
                                            border: Border(
                                                bottom: BorderSide(
                                                    width: 15,
                                                    color: Color(0xffE8ECEF)))),
                                        child: FeedPost(
                                          postData: post,
                                          currentUser: currentUser,
                                          images: images,
                                          likeId: likeId,
                                          dislikeId: dislikeId,
                                          saveId: saveId,
                                        ),
                                      );
                                    },
                                    separatorBuilder:
                                        (BuildContext context, int index) =>
                                            const Divider(),
                                  ),
                                ),
                                if (newPostFlag == true) _newPostButton(),
                                Align(
                                  alignment: Alignment(0.8, 0.85),
                                  child: Container(
                                    decoration: BoxDecoration(
                                      shape: BoxShape.circle,
                                      boxShadow: [
                                        BoxShadow(
                                          color: Colors.grey,
                                          blurRadius: 5.0,
                                          spreadRadius: 1.0,
                                        )
                                      ],
                                    ),
                                    child: ClipOval(
                                      child: Material(
                                        color: Color(0xFF3DD598),
                                        child: InkWell(
                                          splashColor: Color(0xFFFCA64F),
                                          child: SizedBox(
                                            width: 70,
                                            height: 70,
                                            child: Icon(Icons.add,
                                                size: 50.0,
                                                color: Colors.white),
                                          ),
                                          onTap: () {
                                            Navigator.push(
                                              context,
                                              MaterialPageRoute(
                                                  builder: (context) =>
                                                      SharePostPage(
                                                          currentUser:
                                                              currentUser)),
                                            );
                                          },
                                        ),
                                      ),
                                    ),
                                  ),
                                )
                              ],
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
          );
        },
      ),
    );
  }

  _newPostButton() {
    return Container(
        // color: Colors.white,
        height: ScreenUtil().setHeight(40),
        alignment: Alignment.center,
        child: SizedBox(
          height: ScreenUtil().setHeight(30),
          width: ScreenUtil().setHeight(170),
          child: Container(
            decoration: ShapeDecoration(
                color: Color(0xFF3DD598),
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      color: Colors.transparent,
                      width: 1,
                      style: BorderStyle.solid),
                  borderRadius: BorderRadius.circular(25),
                )),
            // ignore: deprecated_member_use
            child: FlatButton(
              onPressed: () {
                this.setState(() {
                  newPostFlag = false;
                });
                _scrollController.animateTo(
                    _scrollController.position.minScrollExtent,
                    duration: Duration(milliseconds: 500),
                    curve: Curves.fastOutSlowIn);
              },
              child: Padding(
                padding: EdgeInsets.only(
                  left: ScreenUtil().setWidth(70),
                ),
                child: Row(
                  children: [
                    Text('New Post', style: TextStyle(color: Colors.white)),
                    Icon(
                      Icons.arrow_forward,
                      size: 35,
                      color: Colors.white,
                    )
                  ],
                ),
              ),
            ),
          ),
        ));
  }

  _buildDateField() {
    final format = DateFormat("dd/MM/yyyy");

    final TextEditingController controller = TextEditingController();
    final now = DateTime.now();

    final nextYear = new DateTime(
      now.year,
      now.month,
      now.day,
    );
    return Container(
        decoration: BoxDecoration(
            border: Border(
          top: BorderSide(width: 15, color: Color(0xffE8ECEF)),
          bottom: BorderSide(width: 15, color: Color(0xffE8ECEF)),
        )),
        height: ScreenUtil().setHeight(70),
        child: Padding(
          padding: EdgeInsets.all(ScreenUtil().setWidth(15)),
          child: Container(
            decoration: BoxDecoration(
                color: Color(0xffE8ECEF),
                borderRadius: BorderRadius.circular(7)),
            margin: EdgeInsets.only(top: ScreenUtil().setHeight(5)),
            child: DateTimeField(
              textAlign: TextAlign.center,
              readOnly: true,
              onChanged: (value) {
                setState(
                  () {
                    selectedValue = value.toString();
                  },
                );
                getPostsListFilter();
              },
              controller: controller,
              format: format,
              onShowPicker: (context, currentValue) {
                return showDatePicker(
                    context: context,
                    firstDate: lastDate,
                    initialDate: currentValue ?? DateTime.now(),
                    lastDate: nextYear);
              },
              decoration: new InputDecoration(
                  labelText: " filter...",
                  labelStyle: TextStyle(color: Color(0xff98A9BC))),
            ),
          ),
        ));
  }

  Future<List<PostModel>> _loadData() async {
    List<PostModel> posts = await feedCubit.feedRepository.loadPosts();
    List<PostModel> sortedPost;
    this.setState(() {
      posts.sort((a, b) => a.createdAt.compareTo(b.createdAt));
      sortedPost = posts.reversed.toList();
      postsListFilter = sortedPost;
      postsList = sortedPost;
    });
    return sortedPost;
  }
}
